#------------------------------------------------------------------------------
# shorthands for MySQL database administration and connections
# afb 5/97
# rev afb 6/98: support of DBI and documentation added
#------------------------------------------------------------------------------

=head1 NAME

Mysql::Admin -- shorthands for MySQL database administration and connections

=head1 SYNOPSIS

   use DBI;
   use Mysql::Admin qw(dbi_connect);

   $db = DBI->connect(dbi_connect($dbdir), "", "");
   $db = DBI->connect(dbi_connect($dbdir, $hostname, $dbname),
                      $login, $passwd);

   # or

   use Mysql;
   use Mysql::Admin qw(setenv);

   setenv($dbdir); $db = Mysql->connect($hostname);
   ($socket, $port) = setenv($dbdir);

   # or

   use Mysql::Admin qw(start stop reload);

   start($dbdir);
   stop($dbdir);
   reload($dbdir);

=head1 DESCRIPTION

I<Mysql::Admin> provides some shorthands for connecting to a MySQL data
base (either by I<DBI> or I<Mysql>) and for administrative operations.
All these shorthands share the assumption that the directory of the
I<MySQL> database contains a file named F<setenv> (format see below)
with the connection parameters.

Usually, F<setenv> will be created by our I<MySQL> installation
procedure (reference is below) that allows to run any number of
I<MySQL> daemons on one system without disturbing each other by using
non-standard locations and non-standard port numbers.

Instead of passing a bunch of parameters the operations of this module
allow to specify your database just be the name of the directory it is
contained in.  This is the I<mysql> subdirectory of the directory
created earlier by our installation procedure.

Following operations are supported:

=over 4

=item dbi_connect

returns a string ready for the first parameter of the DBI->connect
operation. Note that I<dbi_connect> has a side effect by setting
the environment parameters (see below). If the UNIX domain socket
of your I<MySQL> is the only way to connect to it, DBI->connect
will not succeed unless B<MYSQL_UNIX_PORT> has been set by
I<dbi_connect> or I<setenv>. This is a restriction due to the
limited support of the DBD driver module for I<MySQL>.

=item setenv

sets the environment parameters for I<MySQL> in dependence of
the file I<setenv> in I<$dbdir>. I<setenv> returns a
tuple consisting of the socket (path name), and the port number.

=item start

starts the database daemon I<mysqld> for I<$dbdir>.

=item stop

shuts the database daemon I<mysqld> for I<$dbdir>.

=item reload

reloads the database of I<$dbdir>.

=back

=head1 DIAGNOSTICS

I<MySQL::Admin> aborts with following messages to I<Carp::croak> in
case of fatal errors:

=over 4

=item "MYSQL_TCP_PORT parameter in %s missing"

Generated by I<setenv> if B<MYSQL_TCP_PORT> is missing in F<setenv>.

=item "MYSQL_UNIX_PORT parameter in %s missing"

Issued by I<setenv> if B<MYSQL_UNIX_PORT> is missing in F<setenv>.

=item "no databases found in %s"

This error is fired by I<dbi_connect> if no database name was given
and no database (beside the administrative database named I<mysql>)
was found in I<$dbdir>.

=item "unable to fork"

I<start>, I<stop>, or I<reload> was unable to spawn a subprocess.

=item "unable to open %s"

I<setenv> generates this error message if the file F<setenv>
under I<$dbdir> cannot be opened.

=back

=head1 ENVIRONMENT

I<setenv> sets following environment parameters for I<MySQL>:

=over 4

=item B<MYSQL_UNIX_PORT>

Path name of the UNIX domain socket of the database.

=item B<MYSQL_TCP_PORT>

TCP/IP port number of the data base.

=back

=head1 FILES

=over 4

=item F</usr/local/mysql-scripts>

Directory containing some tools for maintaining a I<MySQL> data base
including an installation procedure F<installdb>. Example:

   /usr/local/mysql-scripts/installdb/ /some/where/db

=item F<logfile> (below I<$dbdir>)

Log file used by I<mysqld>.

=item F<setenv> (below I<$dbdir>)

File created by I<createdb> that stores the environment parameters for
I<MySQL>. It contains the settings for B<MYSQL_UNIX_PORT> and
B<MYSQL_TCP_PORT>.
Example (I<$dbdir> is assumed to be F</some/where/db/mysql>):

   MYSQL_UNIX_PORT=/some/where/db/mysql/socket
   MYSQL_TCP_PORT=17169

=item F<socket> (below I<$dbdir>)

UNIX domain socket of the I<MySQL> daemon.

=item F<mysql> (below I<$dbdir>)

Administrative database of I<MySQL>.

=item F</usr/local/mysql>

Directory containing the I<MySQL> software package.

=item F</usr/local/mysql/mysqld>

Binary of the I<MySQL> daemon, invoked by I<start>.

=item F</usr/local/mysql/mysqladmin>

I<reload> and I<stop> are passed to this command.

=back

=head1 SEE ALSO

=over 4

=item I<createdb>

Creates a new database. This tool is installed by the installation
procedure and lives in the F<bin> sub directory (see above).
More about I<createdb> may be found at:

=item Installation Procedure for MySQL

=for html
We have setup a
<A HREF="http://www.mathematik.uni-ulm.de/sai/ws99/uxdb2/mysql.html">page
that shows how to start your own MySQL daemon</A>.

=for text
We have setup a page that shows how to start your own MySQL
daemon: http://www.mathematik.uni-ulm.de/sai/ws99/uxdb2/mysql.html

=back

=head1 AUTHOR

=for html
<A HREF="http://www.mathematik.uni-ulm.de/sai/borchert/">Andreas Borchert</A>

=for text
Andreas Borchert

=head1 BUGS

This module assumes that I<MySQL> has been installed under F</usr/local/mysql>.

Connections to different I<MySQL> databases in parallel could be
troublesome because of the use of environment variables for passing
the connection parameters. This is not a bug of this module but of I<MySQL>.

=cut

#------------------------------------------------------------------------------
package Mysql::Admin;
use strict;
use Carp;
use DirHandle;

require Exporter;
our $VERSION = "0.01";
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(start stop reload setenv databases dbi_connect);

#------------------------------------------------------------------------------
# tunables
#------------------------------------------------------------------------------
my $home = "/usr/local/mysql"; # home directory of mysql distribution
my $bindir = "$home/bin";      # mysqld is located here
my $mysqld = "$bindir/mysqld";
my $mysqladmin = "$bindir/mysqladmin";
my $cmdname = $0; $cmdname =~ s{.*/}{};

sub start {
   my $datadir = shift;

   my($socket, $port) = setenv($datadir);
   my $logfile = "$datadir/logfile";
   my @args = ($mysqld,
      "--basedir=$home",
      "--datadir=$datadir",
      "--port=$port",
      "--socket=$socket",
      "--log=$logfile",
   );
   croak "unable to fork" unless defined spawn(@args);
   sleep(3);
}

sub stop {
   my $datadir = shift;

   setenv($datadir);
   die "$cmdname: unable to shutdown daemon\n"
      unless system($mysqladmin, "shutdown") == 0;
}

sub reload {
   my $datadir = shift;

   setenv($datadir);
   die "$cmdname: unable to reload database\n"
      unless system($mysqladmin, "reload") == 0;
}

sub setenv {
   my $datadir = shift;
   my $paramfile = "$datadir/setenv";
   croak "unable to open $paramfile" unless open(ENV, $paramfile);
   my $socket = undef;
   my $port = undef;
   while(<ENV>) {
      $socket = $1 if /^MYSQL_UNIX_PORT=(.*)$/;
      $port = $1 if /^MYSQL_TCP_PORT=(.*)$/;
   }
   close(ENV);
   croak "MYSQL_UNIX_PORT parameter in $paramfile missing"
      unless defined $socket;
   croak "MYSQL_TCP_PORT parameter in $paramfile missing"
      unless defined $port;
   $ENV{'MYSQL_UNIX_PORT'} = $socket;
   $ENV{'MYSQL_TCP_PORT'} = $port;
   return ($socket, $port);
}

sub dbi_connect {
   my ($datadir, $hostname, $db) = @_;
   my ($socket, $port) = setenv($datadir);
   $hostname = "" unless defined $hostname;
   unless (defined $db) {
      my @databases = databases($datadir);
      croak "no databases found in $datadir" unless @databases >= 2;
      $db = shift @databases;
      $db = shift @databases if $db eq "mysql";
   }
   return "DBI:mysql:$db:$hostname:$port";
}

sub databases {
   my $datadir = shift;
   my $dir = new DirHandle $datadir;
   return () unless defined $dir;
   my @databases = ();
   my $file;
   while (defined($file = $dir->read())) {
      next if $file =~ /^\./;
      push (@databases, $file) if -d "$datadir/$file";
   }
   return sort @databases;
}

sub spawn {
   my @args = @_;
   my $pid;
   unless ($pid = fork) {
      exec(@args);
      die "$cmdname: unable to exec " . join(" ", @args) . "\n";
   }
   return $pid;
}

1;
